#include <Adafruit_NeoPixel.h>
#include "U8glib.h"
#include <Wire.h>
#include "RTClib.h"
RTC_DS1307 RTC;
int UVOUT = A0;           //Output from the sensor
int REF_3V3 = A1;         //3.3V power on the Arduino board
const int buzzerpin = 4;  //Connect the buzzer
#define PIN 5             //Connect the RGB-LED
#define time 20          //Set the RGB-LED working time
#define uvLevel_Threshold 200  //set the buzzer alarm value of the uvLevel
int uvLevel =0;
unsigned int year,month,day,hour,minute,second;
U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NONE);
Adafruit_NeoPixel strip = Adafruit_NeoPixel(60, PIN, NEO_GRB + NEO_KHZ800);

void setup(void) {
  Serial.begin(9600);
  pinMode(UVOUT, INPUT);
  pinMode(REF_3V3, INPUT);
  pinMode(buzzerpin,OUTPUT);
  digitalWrite(buzzerpin,LOW);
  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
  Wire.begin();
  RTC.begin();
  if (! RTC.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    RTC.adjust(DateTime(__DATE__, __TIME__));
  }
}

void loop(void)
{
    u8g.firstPage();  //OLED Display
    do {
       OLED_display();
       u8g.setColorIndex(1);
      } while( u8g.nextPage() );
   if(hour>=time)
   {
      RGBLED_display();
   }
   if(uvLevel>uvLevel_Threshold)
   {
       buzzer();
   }
}

void OLED_display(void) 
{
     uvLevel = averageAnalogRead(UVOUT);
     RTC_get();
     u8g.setFont(u8g_font_7x14);
     u8g.setPrintPos(10, 10);
     u8g.print(year, DEC);
     u8g.print("/");
     u8g.print(month,DEC);
     u8g.print("/");          
     u8g.print(day,DEC);
     delay(1);
     u8g.setPrintPos(10,25);            
     u8g.print(hour,DEC);
     u8g.print(":"); 
     u8g.print(minute,DEC);
     u8g.print(":"); 
     u8g.print(second,DEC);
     delay(1);        
     u8g.setPrintPos(10,40); 
     u8g.print("UVLevel:");        
     u8g.print(uvLevel,DEC);
}
void RGBLED_display(void)
{
  colorWipe(strip.Color(255, 0, 0), 5); // Red
  colorWipe(strip.Color(0, 255, 0), 5); // Green
  colorWipe(strip.Color(0, 0, 255), 5); // Blue
}
void RTC_get(void)
{
    DateTime now = RTC.now();
    year=now.year();
    month=now.month();
    day=now.day();
    hour=now.hour();
    minute=now.minute();
    second=now.second();
}
int averageAnalogRead(int pinToRead)
{
  byte numberOfReadings = 8;
  unsigned int runningValue = 0; 
  for(int x = 0 ; x < numberOfReadings ; x++)
    runningValue += analogRead(pinToRead);
  runningValue /= numberOfReadings;
  return(runningValue);  
}

void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, c);
      strip.show();
      delay(wait);
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  if(WheelPos < 85) {
   return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else {
   WheelPos -= 170;
   return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}
void buzzer(void)
{
  for(int i=0;i<5;i++)
  {
    digitalWrite(buzzerpin,HIGH);
    delay(500);
    digitalWrite(buzzerpin,LOW);
    delay(500);  
  } 
}
